/*
 * Copyright (C) 1999  Renato Mezzena
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 */


/***************************************************************************
 *
 *  adcconvert.c
 *
 *  This program is used to sample data with a PCI-das1602 card; a FFT
 *  routine is called and the frequency domain data are sent to the stdout
 *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include "pci-das1602.h"


/***************************************************************************
 *
 *  Global Data
 *
 ***************************************************************************/

char *DevName = "/dev/pci-das1602-16_adc00";
int  Gain ;
int  ADC_Mode     = ADC_PACER_CLOCK;
int  np ;
int  freq_A2D ;
int flagmed ;
int fdADC;                   /* A/D file descriptors */
double norm,totgain ;

void DoOpenDevices()
{

  char str[80];

  if (( fdADC = open(DevName, ADC_Mode)) < 0 ) {
    perror(str);
    printf("error opening device %s\n", DevName);
    exit(2);
  }

}


void ADC()
{
  WORD value[16384];
  int i,j,nmed ;
  int bytesRead;
  double realsmp[16384],medie[8192];
  double pi=3.1415927;

  realsmp[np]=0.0;
  nmed=0;
  j=1;

  while ( j = 1) {   
     bytesRead = read(fdADC, value, np);

    if ( bytesRead != np ) {
      printf("testADC: Error on read() \n");
    }

    for ( i = 0; i < np; i++ ) {
      realsmp[i]=(value[i]-32767.5) *10.0/32767.5/Gain/totgain ;
      realsmp[i]*= 1.63 *(0.5 -0.5*cos(2.0*pi*i/(np- 1.0)));

    }

    realfft(np, realsmp);

    if ( flagmed ==1) {
      nmed++ ;
      for (i=0;i<np/2;i++) {
        medie[i]+= norm*(pow(realsmp[i],2)+pow(realsmp[np-i],2)) ;
        printf("%e\n",sqrt(medie[i]/nmed));
      }
    } else {
      for (i=0;i<np/2;i++) {
        printf("%e\n",sqrt(norm)*(hypot(realsmp[i],realsmp[np-i])));
      }
    }
  }

}


main()
{

   DoOpenDevices();

   fscanf(stdin,"%d",&np);
   fscanf(stdin,"%d",&freq_A2D);
   fscanf(stdin,"%d",&Gain);
   fscanf(stdin,"%d",&flagmed);
   fscanf(stdin,"%lf",&totgain); 
 
    switch(Gain){
    case 1:
      ioctl(fdADC, ADC_SET_GAINS, BP_10_00V);
      break;
    case 2:
      ioctl(fdADC, ADC_SET_GAINS, BP_5_00V);
      break;
    case 4:
      ioctl(fdADC, ADC_SET_GAINS, BP_2_50V);
      break;
    case 8:
      ioctl(fdADC, ADC_SET_GAINS, BP_1_25V);
      break;
    }
  ioctl(fdADC, ADC_SET_FRONT_END,"FALSE");
  ioctl(fdADC, ADC_STOP_PACER, 0);
  ioctl(fdADC, ADC_SET_PACER_FREQ, freq_A2D);
  ioctl(fdADC, ADC_GET_PACER_FREQ, &freq_A2D);
  ioctl(fdADC, ADC_START_PACER, 0);
  printf("%d\n", freq_A2D); 
  norm=2.0/(freq_A2D*np) ;

  ADC(); 
  close(fdADC);
}


